/*
 * $Id$
 */

package org.sonatype.nexus.repository.metadata.model;

  //---------------------------------/
 //- Imported classes and packages -/
//---------------------------------/

import java.util.Date;

/**
 * The root element of Repository Metadata.
 * 
 * @version $Revision$ $Date$
 */
public class RepositoryMetadata implements java.io.Serializable {


      //--------------------------/
     //- Class/Member Variables -/
    //--------------------------/

    /**
     * The version of the persisted model.
     */
    private String version;

    /**
     * The repository "canonical url".
     */
    private String url;

    /**
     * The repository recommended ID.
     */
    private String id;

    /**
     * The repository recommended name.
     */
    private String name;

    /**
     * The repository "layout".
     */
    private String layout;

    /**
     * The repository "policy".
     */
    private String policy;

    /**
     * The repository "local url".
     */
    private String localUrl;

    /**
     * The repository public key.
     */
    private String publicKey;

    /**
     * The email of the repository maintainer.
     */
    private String repositoryMaintainerEmail;

    /**
     * Simple UTC timestamp of last content change in Repository.
     * MRM/IDEs may use this value to
     *             flush their caches.
     *           
     */
    private long lastContentChange = 0;

    /**
     * A typical period of content change in repository, if any. In
     * minutes. For example: the period
     *             when sync tools are
     *             run. MRM/IDEs may use this value to tweak their
     * cache TTLs.
     */
    private int contentChangeInterval = 0;

    /**
     * The dynamic source of repository mirror list (formed based
     * on request).
     */
    private String mirrorListSource;

    /**
     * Field mirrors.
     */
    private java.util.List mirrors;

    /**
     * Field memberRepositories.
     */
    private java.util.List memberRepositories;


      //-----------/
     //- Methods -/
    //-----------/

    /**
     * Method addMemberRepository.
     * 
     * @param repositoryMemberMetadata
     */
    public void addMemberRepository(RepositoryMemberMetadata repositoryMemberMetadata)
    {
        if ( !(repositoryMemberMetadata instanceof RepositoryMemberMetadata) )
        {
            throw new ClassCastException( "RepositoryMetadata.addMemberRepositories(repositoryMemberMetadata) parameter must be instanceof " + RepositoryMemberMetadata.class.getName() );
        }
        getMemberRepositories().add( repositoryMemberMetadata );
    } //-- void addMemberRepository(RepositoryMemberMetadata) 

    /**
     * Method addMirror.
     * 
     * @param repositoryMirrorMetadata
     */
    public void addMirror(RepositoryMirrorMetadata repositoryMirrorMetadata)
    {
        if ( !(repositoryMirrorMetadata instanceof RepositoryMirrorMetadata) )
        {
            throw new ClassCastException( "RepositoryMetadata.addMirrors(repositoryMirrorMetadata) parameter must be instanceof " + RepositoryMirrorMetadata.class.getName() );
        }
        getMirrors().add( repositoryMirrorMetadata );
    } //-- void addMirror(RepositoryMirrorMetadata) 

    /**
     * Get a typical period of content change in repository, if
     * any. In minutes. For example: the period
     *             when sync tools are
     *             run. MRM/IDEs may use this value to tweak their
     * cache TTLs.
     * 
     * @return int
     */
    public int getContentChangeInterval()
    {
        return this.contentChangeInterval;
    } //-- int getContentChangeInterval() 

    /**
     * Get the repository recommended ID.
     * 
     * @return String
     */
    public String getId()
    {
        return this.id;
    } //-- String getId() 

    /**
     * Get simple UTC timestamp of last content change in
     * Repository. MRM/IDEs may use this value to
     *             flush their caches.
     *           
     * 
     * @return long
     */
    public long getLastContentChange()
    {
        return this.lastContentChange;
    } //-- long getLastContentChange() 

    /**
     * Get the repository "layout".
     * 
     * @return String
     */
    public String getLayout()
    {
        return this.layout;
    } //-- String getLayout() 

    /**
     * Get the repository "local url".
     * 
     * @return String
     */
    public String getLocalUrl()
    {
        return this.localUrl;
    } //-- String getLocalUrl() 

    /**
     * Method getMemberRepositories.
     * 
     * @return java.util.List
     */
    public java.util.List getMemberRepositories()
    {
        if ( this.memberRepositories == null )
        {
            this.memberRepositories = new java.util.ArrayList();
        }
        
        return this.memberRepositories;
    } //-- java.util.List getMemberRepositories() 

    /**
     * Get the dynamic source of repository mirror list (formed
     * based on request).
     * 
     * @return String
     */
    public String getMirrorListSource()
    {
        return this.mirrorListSource;
    } //-- String getMirrorListSource() 

    /**
     * Method getMirrors.
     * 
     * @return java.util.List
     */
    public java.util.List getMirrors()
    {
        if ( this.mirrors == null )
        {
            this.mirrors = new java.util.ArrayList();
        }
        
        return this.mirrors;
    } //-- java.util.List getMirrors() 

    /**
     * Get the repository recommended name.
     * 
     * @return String
     */
    public String getName()
    {
        return this.name;
    } //-- String getName() 

    /**
     * Get the repository "policy".
     * 
     * @return String
     */
    public String getPolicy()
    {
        return this.policy;
    } //-- String getPolicy() 

    /**
     * Get the repository public key.
     * 
     * @return String
     */
    public String getPublicKey()
    {
        return this.publicKey;
    } //-- String getPublicKey() 

    /**
     * Get the email of the repository maintainer.
     * 
     * @return String
     */
    public String getRepositoryMaintainerEmail()
    {
        return this.repositoryMaintainerEmail;
    } //-- String getRepositoryMaintainerEmail() 

    /**
     * Get the repository "canonical url".
     * 
     * @return String
     */
    public String getUrl()
    {
        return this.url;
    } //-- String getUrl() 

    /**
     * Get the version of the persisted model.
     * 
     * @return String
     */
    public String getVersion()
    {
        return this.version;
    } //-- String getVersion() 

    /**
     * Method removeMemberRepository.
     * 
     * @param repositoryMemberMetadata
     */
    public void removeMemberRepository(RepositoryMemberMetadata repositoryMemberMetadata)
    {
        if ( !(repositoryMemberMetadata instanceof RepositoryMemberMetadata) )
        {
            throw new ClassCastException( "RepositoryMetadata.removeMemberRepositories(repositoryMemberMetadata) parameter must be instanceof " + RepositoryMemberMetadata.class.getName() );
        }
        getMemberRepositories().remove( repositoryMemberMetadata );
    } //-- void removeMemberRepository(RepositoryMemberMetadata) 

    /**
     * Method removeMirror.
     * 
     * @param repositoryMirrorMetadata
     */
    public void removeMirror(RepositoryMirrorMetadata repositoryMirrorMetadata)
    {
        if ( !(repositoryMirrorMetadata instanceof RepositoryMirrorMetadata) )
        {
            throw new ClassCastException( "RepositoryMetadata.removeMirrors(repositoryMirrorMetadata) parameter must be instanceof " + RepositoryMirrorMetadata.class.getName() );
        }
        getMirrors().remove( repositoryMirrorMetadata );
    } //-- void removeMirror(RepositoryMirrorMetadata) 

    /**
     * Set a typical period of content change in repository, if
     * any. In minutes. For example: the period
     *             when sync tools are
     *             run. MRM/IDEs may use this value to tweak their
     * cache TTLs.
     * 
     * @param contentChangeInterval
     */
    public void setContentChangeInterval(int contentChangeInterval)
    {
        this.contentChangeInterval = contentChangeInterval;
    } //-- void setContentChangeInterval(int) 

    /**
     * Set the repository recommended ID.
     * 
     * @param id
     */
    public void setId(String id)
    {
        this.id = id;
    } //-- void setId(String) 

    /**
     * Set simple UTC timestamp of last content change in
     * Repository. MRM/IDEs may use this value to
     *             flush their caches.
     *           
     * 
     * @param lastContentChange
     */
    public void setLastContentChange(long lastContentChange)
    {
        this.lastContentChange = lastContentChange;
    } //-- void setLastContentChange(long) 

    /**
     * Set the repository "layout".
     * 
     * @param layout
     */
    public void setLayout(String layout)
    {
        this.layout = layout;
    } //-- void setLayout(String) 

    /**
     * Set the repository "local url".
     * 
     * @param localUrl
     */
    public void setLocalUrl(String localUrl)
    {
        this.localUrl = localUrl;
    } //-- void setLocalUrl(String) 

    /**
     * Set the list of repositories participating in this group.
     * 
     * @param memberRepositories
     */
    public void setMemberRepositories(java.util.List memberRepositories)
    {
        this.memberRepositories = memberRepositories;
    } //-- void setMemberRepositories(java.util.List) 

    /**
     * Set the dynamic source of repository mirror list (formed
     * based on request).
     * 
     * @param mirrorListSource
     */
    public void setMirrorListSource(String mirrorListSource)
    {
        this.mirrorListSource = mirrorListSource;
    } //-- void setMirrorListSource(String) 

    /**
     * Set the static list of repository mirrors.
     * 
     * @param mirrors
     */
    public void setMirrors(java.util.List mirrors)
    {
        this.mirrors = mirrors;
    } //-- void setMirrors(java.util.List) 

    /**
     * Set the repository recommended name.
     * 
     * @param name
     */
    public void setName(String name)
    {
        this.name = name;
    } //-- void setName(String) 

    /**
     * Set the repository "policy".
     * 
     * @param policy
     */
    public void setPolicy(String policy)
    {
        this.policy = policy;
    } //-- void setPolicy(String) 

    /**
     * Set the repository public key.
     * 
     * @param publicKey
     */
    public void setPublicKey(String publicKey)
    {
        this.publicKey = publicKey;
    } //-- void setPublicKey(String) 

    /**
     * Set the email of the repository maintainer.
     * 
     * @param repositoryMaintainerEmail
     */
    public void setRepositoryMaintainerEmail(String repositoryMaintainerEmail)
    {
        this.repositoryMaintainerEmail = repositoryMaintainerEmail;
    } //-- void setRepositoryMaintainerEmail(String) 

    /**
     * Set the repository "canonical url".
     * 
     * @param url
     */
    public void setUrl(String url)
    {
        this.url = url;
    } //-- void setUrl(String) 

    /**
     * Set the version of the persisted model.
     * 
     * @param version
     */
    public void setVersion(String version)
    {
        this.version = version;
    } //-- void setVersion(String) 


    public static final String MODEL_VERSION = "1.0.0";

    public static final String LAYOUT_MAVEN2 = "maven2";
    public static final String LAYOUT_MAVEN1 = "maven1";

    public static final String POLICY_RELEASE = "release";
    public static final String POLICY_SNAPSHOT = "snapshot";
    public static final String POLICY_MIXED = "mixed";

    private String modelEncoding = "UTF-8";

    /**
     * Set an encoding used for reading/writing the model.
     *
     * @param modelEncoding the encoding used when reading/writing the model.
     */
    public void setModelEncoding( String modelEncoding )
    {
        this.modelEncoding = modelEncoding;
    }

    /**
     * @return the current encoding used when reading/writing this model.
     */
    public String getModelEncoding()
    {
        return modelEncoding;
    }
}
